public abstract class Sequence<E> {
    // Overview: Tipo delle sequenze finite di oggetti di tipo E di dimensioni e contenuto
    // modificabili i cui elementi possono essere null
    // Typical Element: obj_0, ..., obj_n con obj_i oggetto di E
    
    /* restituisce la lunghezza della sequenza */
    public abstract int length();
    /*
     @effects: restituisce il valore n+1 per un oggetto obj_0, ..., obj_n, e 0 altrimenti
     */
    
    /* restituisce la prima posizione nella sequenza il cui elemento corrispondente e'
     uguale a elem */
    public abstract int indexOf(E elem);
    /*
    @ effects: restituisce i tale che obj_i = null se elem == null o obj_i.equals(elem) 
     altrimenti e forall k. k < i -> ! obj_k.equals(elem), e -1 se elem non e' presente
     */
    
    /* restituisce la prima posizione nella sequenza a partire dalla quale gli elementi 
     successivi formano la sequenza s */
    public abstract int indexOfSlide(Sequence<E> s) throws NullPointerException;
    /*
     @effects: restituisce i tale che P(i) && forall m. m < i -> ! P(m), dove
     P(x) = forall k. x <= k < x+s.length() -> ((obj_k == null -> s_{k-x} == null) ||
                                                obj_k.equals(s_{k-x})),
     e -1 se la sotto-sequenza s non e' presente.
     @throws Se s == null lancia una NullPointerException (eccezione disponibile in Java, unchecked)
     */
    
    /* modifica la sequenza inserendo alla posizione pos l'oggetto elem */
    public abstract void update(int pos, E elem);
    /*
     MODIFIES: this
     EFFECTS: modifica la sequenza eseguendo obj_pos = elem.
     Se pos < 0 lancia una IllegalArgumentException (eccezione disponibile in Java, unchecked)
     */
   
    /* controlla se la sequenza iniziale coincide con il parametro s */
    public boolean checkInitial(Sequence<E> s) {
        return this.indexOfSlide(s) == 0;
    }
    /*
     @modifies this
     @effects: restituisce this.indexOfSlide(s) == 0.
     @Throws Se s == null lancia una NullPointerException (eccezione disponibile in Java, unchecked)
     */

    /* restituisce l'elemento della sequenza in posizione i */
    public abstract E get(int pos) throws IllegalArgumentException;
    /*
     @effects: restituisce obj_pos se 0 <= pos < this.length() e null se pos >= this.length().
     @throws: Se pos < 0 lancia una IllegalArgumentException (eccezione disponibile in Java, unchecked)
     */
}
